<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class Event implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation");
  private $validation;
  
  private $event_id;
  private $active;
  private $db_admin_group_id;
  private $event_name;
  private $event_title;
  private $event_location;
  private $event_period;
  private $event_email;
  private $event_description;
  private $disclaimer_id;
  private $allow_multiple_reg;
  private $enable_waitlist;
  private $max_abstracts;
  private $confirmation_text;
  private $confirmation_email;
  private $confirmation_text_wl;
  private $confirmation_email_wl;
  private $reg_email;
  private $jump2url;
  private $reg_start;
  private $reg_start_txt;
  private $reg_end;
  private $reg_end_txt;
  private $external_css;
  
  private $event_status;
  private $last_deadline;
  private $has_form;

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    $this->validation = $validation;
  }

  public function __clone() {
    $this->validation = null;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if($name == "id") $name = "event_id";
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
      
    return null;
  }

  public function getIDName() {
    return "event_id";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray))
        $array[$key] = $value; 
    return $array;
  }
  
  public function getStatus() {
    if (!$this->has_form)
      return _("No registration form");
    switch ($this->event_status) {
      case EVENT_NOT_STARTED:   return $this->reg_start_txt; break;
      case EVENT_ENDED:         return $this->reg_end_txt; break;
      case EVENT_PAST_DEADLINE: return _("booking deadline exceeded (deadline: ").CWFTools::dateISO2Local($this->last_deadline, getenv("LC_ALL")).")"; break;
      case EVENT_BOOKED_OUT:    return _("Booked out"); break;
      case EVENT_WAITLIST:      return _("Booked out")." ("._("waiting list possible").")"; break;
      case EVENT_REGISTERED:    return _("registered"); break;
      default:                  return _("booking possible"); break;
    }
    return "";
  }
  
  public function getPostBackData() {
    return array($this->getIDName()=>$this->id);
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
        case "event_status":
        case "max_deadline":
        case "has_form":
          break;
          
        case "db_admin_group_id":
          if(isset($array[$key]) && preg_match('~^[0-9]+$~', $array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
          
        case "active":
        case "allow_multiple_reg":
        case "enable_waitlist":
          $this->$key = ((isset($array[$key]) && $array[$key]) ? 1 : 0);
          break;
          
        case "reg_start":
        case "reg_end":
          if(isset($array[$key]))
            $this->$key = CWFTools::dateLocal2ISO($array[$key], getenv("LC_ALL"));
          else
            $this->$key = null;
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
      }
    }
    return $this->validation->validate($array);
  }
  
  private function fromDbArray($array) {
    foreach ($array as $property => $value) {
      if(property_exists($this, $property)) {
        switch ($property) {
          default:
            $this->$property = $value;
            break;
        }
      }
    }
  } 
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  static private function queryEvents(DbConnection $DB, $conditions, &$error) {
    $sql = "SELECT e.*,\n".
           "       CASE ".(isset($_SESSION['user'])
                     ? "\nWHEN (SELECT count(*) FROM user_booking ub WHERE ub.event_id=e.event_id AND ub.user_id='{$_SESSION['user']->user_id}' AND ub.deleted=0)>0 THEN ".EVENT_REGISTERED
                     : "\n" ) .
           "         WHEN ifnull(e.reg_start, '0000-00-00')>NOW() THEN ".EVENT_NOT_STARTED."\n".
           "         WHEN ifnull(e.reg_end, '9999-12-31')<NOW() THEN ".EVENT_ENDED."\n".
           "         WHEN MAX(ebs.past_deadline)>0 THEN ".EVENT_PAST_DEADLINE."\n".
           "         WHEN MAX(ebs.option_closed*ebs.enable_waitlist)>0 THEN ".EVENT_WAITLIST."\n".
           "         WHEN MAX(ebs.option_closed)>0 THEN ".EVENT_BOOKED_OUT."\n".
           "         ELSE ".EVENT_OPEN."\n".
           "       END event_status,\n".
           "       MIN(ebs.fee_last_deadline) last_deadline,\n".
           "       IF((SELECT COUNT(*)\n".
           "           FROM event_option_groups eogss\n". 
           "             JOIN event_options eoss ON eoss.event_option_group_id=eogss.event_option_group_id\n".
           "           WHERE eogss.event_id=e.event_id)>0, 1, 0) has_form\n".
           "FROM events e\n".
           "  LEFT JOIN\n".
           "   (SELECT eog.event_id,\n".
           "           (max(ifnull(ef.deadline, '9999-12-31'))<NOW()) past_deadline,\n".
           "           ((eo.person_limit>0 AND eo.person_limit<=(SELECT ifnull(sum(ubo.value_int), 0) FROM user_booking_options ubo WHERE ubo.event_option_id=eo.event_option_id AND ubo.deleted=0))\n". 
           "             OR ef.deadline<NOW()) option_closed,\n".
           "           eog.enable_waitlist,\n".
           "           ef.deadline fee_last_deadline\n".
           "    FROM event_option_groups eog\n".
           "      LEFT JOIN event_options eo ON (eo.event_option_group_id=eog.event_option_group_id)\n".
           "      LEFT JOIN (SELECT event_option_id, max(ifnull(deadline, '9999-12-31')) deadline FROM event_fees GROUP BY event_option_id) ef ON ef.event_option_id=eo.event_option_id\n".
           "    WHERE eog.group_type IN ('".OPTION_GROUP_FEES."')\n".
           "      AND eog.required=1\n".
           "      AND eog.exclusive=0\n".
           "    GROUP BY eo.event_option_id\n".
           "    UNION\n".
           "    SELECT eog.event_id,\n".
           "           (max(ifnull(ef.deadline, '9999-12-31'))<NOW()) past_deadline,\n".
           "           min((eo.person_limit>0 AND eo.person_limit<=(SELECT ifnull(sum(ubo.value_int), 0) FROM user_booking_options ubo WHERE ubo.event_option_id=eo.event_option_id AND ubo.deleted=0))\n".
           "             OR ef.deadline<NOW()) option_closed,\n".
           "           eog.enable_waitlist,\n".
           "           ef.deadline fee_last_deadline\n".
           "    FROM event_option_groups eog\n".
           "      LEFT JOIN event_options eo ON (eo.event_option_group_id=eog.event_option_group_id)\n".
           "      LEFT JOIN (SELECT event_option_id, max(ifnull(deadline, '9999-12-31')) deadline FROM event_fees GROUP BY event_option_id) ef ON ef.event_option_id=eo.event_option_id\n".
           "    WHERE ((eog.group_type='".OPTION_GROUP_FEES."' AND eog.exclusive=1) OR eog.group_type='".OPTION_GROUP_TICKETS."')\n".
           "      AND eog.required=1\n".
           "    GROUP BY eog.event_option_group_id) ebs ON ebs.event_id=e.event_id\n".
           ($conditions ? "WHERE $conditions" : "")."\n".
           "GROUP BY e.event_id\n".
           "ORDER BY e.event_name;";

    $result = $DB->query($sql);

    if(!$DB->errno() && $DB->num_rows($result)>0)
      return $result;
    else {
      $error = array($DB->errno(), $DB->error());
      return false;
    }
  }
  
  public function readData(DbConnection $DB, $conditions) {
    $whereConditions = array();
    foreach ($conditions as $key => $value)
      if(array_key_exists($key, get_class_vars(get_class())))
        $whereConditions[] = "e.$key=$value";
    
    if($result = self::queryEvents($DB, implode(" AND ", $whereConditions), $error)) {
      $event = $DB->fetch_array($result);
      $this->fromDbArray($event);
      
      return array(0, "");
    }
    else
      return $error;
  }
  
  static public function readList(DbConnection $DB) {
    $result = $DB->query("SELECT event_id id, event_name name, active, db_admin_group_id FROM events ORDER BY active DESC, event_name;");
    
    $list = array();
    while ($event = $DB->fetch_array($result))
      $list[] = $event;
      
    return $list;
  }

  static public function getList(DbConnection $DB, $conditions, &$error="") {
    $whereConditions = array();
    foreach ($conditions as $key => $value)
      if(array_key_exists($key, get_class_vars(get_class())))
        $whereConditions[] = "e.$key=$value";
    
    if($result = self::queryEvents($DB, implode(" AND ", $whereConditions), $error)) {
      while($eventArray = $DB->fetch_array($result)) {
        $event = new Event();
        $event->fromDbArray($eventArray);

        $eventList[$event->event_id] = $event;
      }
      
      return $eventList;
    }
    else {
      return false;
    }
  }
  
  static public function getBookedEvents(DbConnection $DB, $userId) {
    $sql = "SELECT e.event_id, e.event_title, e.event_location, e.event_period, e.event_email, e.event_description
            FROM events e
              JOIN user_booking ub ON (ub.event_id=e.event_id AND ub.deleted=0) 
            WHERE e.active=0
              AND ub.user_id={$userId}
            ORDER BY e.event_title;";
    $result = $DB->query($sql);
    
    $list = array();
    while ($event = $DB->fetch_array($result))
      $list[] = $event;
      
    return $list;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write

  public function writeData(DbConnection $DB) {
    if(!isset($this->event_name) || !$this->event_name) return;
    if(isset($this->event_id) && $this->event_id) {
      $result = $DB->query("UPDATE events SET
          db_admin_group_id=".(!$this->db_admin_group_id ? "null" : "'{$this->db_admin_group_id}'").",
          event_name='{$this->event_name}',
          event_title='{$this->event_title}',
          event_location='{$this->event_location}',
          event_period='{$this->event_period}',
          event_email='{$this->event_email}',
          event_description='{$this->event_description}',
          disclaimer_id='{$this->disclaimer_id}',
          allow_multiple_reg='{$this->allow_multiple_reg}',
          enable_waitlist='{$this->enable_waitlist}',
          max_abstracts='{$this->max_abstracts}',
          confirmation_text='{$this->confirmation_text}',
          confirmation_email='{$this->confirmation_email}',
          confirmation_text_wl='{$this->confirmation_text_wl}',
          confirmation_email_wl='{$this->confirmation_email_wl}',
          reg_email='{$this->reg_email}',
          jump2url='{$this->jump2url}',
          reg_start=".(!$this->reg_start ? "null" : "'{$this->reg_start}'").",
          reg_start_txt='{$this->reg_start_txt}',
          reg_end=".(!$this->reg_end ? "null" : "'{$this->reg_end}'").",
          reg_end_txt='{$this->reg_end_txt}',
          external_css='{$this->external_css}'
        WHERE event_id='{$this->event_id}';");
      
      $logEntry = "Event '{$this->event_id}' changed.";
    }
    else {
      $result = $DB->query("INSERT INTO events (
          active, db_admin_group_id, event_name, event_title, 
          event_location, event_period, event_email, event_description, 
          disclaimer_id, allow_multiple_reg, enable_waitlist, max_abstracts, 
          confirmation_text, confirmation_email, confirmation_text_wl, confirmation_email_wl, 
          reg_email, jump2url, reg_start, reg_start_txt, reg_end, reg_end_txt, external_css
        ) VALUES (
          1,
          ".(!$this->db_admin_group_id ? "null" : "'{$this->db_admin_group_id}'").",
          '{$this->event_name}',
          '{$this->event_title}',
          '{$this->event_location}',
          '{$this->event_period}',
          '{$this->event_email}',
          '{$this->event_description}',
          '{$this->disclaimer_id}',
          '{$this->allow_multiple_reg}',
          '{$this->enable_waitlist}',
          '{$this->max_abstracts}',
          '{$this->confirmation_text}',
          '{$this->confirmation_email}',
          '{$this->confirmation_text_wl}',
          '{$this->confirmation_email_wl}',
          '{$this->reg_email}',
          '{$this->jump2url}',
          ".(!$this->reg_start ? "null" : "'{$this->reg_start}'").",
          '{$this->reg_start_txt}',
          ".(!$this->reg_end ? "null" : "'{$this->reg_end}'").",
          '{$this->reg_end_txt}',
          '{$this->external_css}');");

      $this->event_id = $DB->insert_id();
      $logEntry = "Event '{$this->event_id}' created.";
    }
    if(!$DB->errno()) {
      KDbLog::writeLog($DB, $logEntry);
      return array(0, "");
    }
    else
      return array($DB->errno(), $DB->error());
  }

  static public function deleteData(DbConnection $DB, $id) {
    if(preg_match("~^[0-9]+$~", $id))
      $condition = "event_id='{$id}'";
    else
      $condition = $id;
      
    $eventIds = $DB->result($DB->query("SELECT group_concat(event_id SEPARATOR ', ') 
                                         FROM events 
                                         WHERE $condition;"), 0, 0);
    $result = $DB->query("DELETE FROM events WHERE {$condition};");
    
    if(!$DB->errno() && $DB->affected_rows()>0) {
      KDbLog::writeLog($DB, "Event(s) deleted ({$condition}).");
      EventOptionsGroup::deleteData($DB, "event_id IN ({$eventIds})");
      UserBooking::deleteData($DB, "event_id IN ({$eventIds})");
      return array(0, "");
    }
    else 
      return array($DB->errno(), $DB->error());
  }
  
  public function setActive(DbConnection $DB) {
    if(!$this->event_id || ($_SESSION['admin']['login']!= "root" && $this->db_admin_group_id && !in_array($this->db_admin_group_id, $_SESSION['admin']['db_admin_group_ids'])))
      return array(1, _("Operation not allowed.")."ID: {$this->event_id} GroupID: {$this->db_admin_group_id}");
    
    $this->active = abs($this->active - 1);
    $result = $DB->query("UPDATE events SET
        active=abs(active-1)
      WHERE event_id='{$this->event_id}';");

    return array($DB->errno(), $DB->error());
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - misc
  static public function createTable(DbConnection $DB) {
    $result = $DB->query("
      CREATE TABLE events (
        event_id int unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
        active tinyint,
        db_admin_group_id int unsigned,
        event_name text NOT NULL,
        event_title text NOT NULL,
        event_location text,
        event_period text,
        event_email text,
        event_description text,
        disclaimer_id int unsigned,
        allow_multiple_reg tinyint,
        enable_waitlist tinyint,
        max_abstracts tinyint,
        confirmation_text text,
        confirmation_email text,
        confirmation_text_wl text,
        confirmation_email_wl text,
        reg_email text,
        jump2url text,
        reg_start date,
        reg_start_txt text,
        reg_end date,
        reg_end_txt text,
        external_css text
      );");
    if ($result) { echo _("&nbsp;&nbsp;- creating table").": 'events'.<br>\n"; }
    else { echo _("&nbsp;&nbsp;<b>- Error when creating table</b>")." 'events'<br>\n"; $error = 1; }
  }
}
?>