<?php
require_once("cwfUI.inc.php");

class EventUI extends cwfUI {
  protected static $accessRight = "edit_event";
  protected static $dataClass = "Event";
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct() {
    parent::__construct();
    $this->initialize();
  }
  
  protected function initialize() {
    parent::initialize();
    $this->title = _("Events");
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  public function getPostBackXML(DbConnection $DB, $action) {
    switch ($action) {
      case "saveasnew":
      case "save":
        $xml = $this->getNewValuesXML($this->data->getPostBackData());
        $_SESSION['event'] = clone $this->data;
        $menu = new MainAdminUI($DB);
        $xml .= "  <replaceHTMLElement id=\"ul#dropdownmenu\"><![CDATA[\n";
        $xml .= $menu->getMenuHTML($DB, "index.php/event");
        $xml .= "]]>\n";
        $xml .= "  </replaceHTMLElement>\n";
        $xml .= "  <runFunction>kdbSetupMenu</runFunction>\n";
        $xml .= "  <runFunction>setupTooltips</runFunction>\n";
        $xml .= "  <activateButtons>\n";
        $xml .= "    <button>continue_event_forms</button>\n";
        $xml .= "  </activateButtons>\n";
        break;
      case "delete":
        $xml = $this->getNewValuesXML($this->getData());
        unset($_SESSION['event']);
        $menu = new MainAdminUI($DB);
        $xml .= "  <replaceHTMLElement id=\"ul#dropdownmenu\"><![CDATA[\n";
        $xml .= $menu->getMenuHTML($DB, "index.php/event");
        $xml .= "]]>\n";
        $xml .= "  </replaceHTMLElement>\n";
        $xml .= "  <runFunction>kdbSetupMenu</runFunction>\n";
        $xml .= "  <runFunction>setupTooltips</runFunction>\n";
        $xml .= "  <deactivateButtons>\n";
        $xml .= "    <button>continue_event_forms</button>\n";
        $xml .= "  </deactivateButtons>\n";
        break;
      default:
        $xml = parent::getPostBackXML($DB, $action);
    }
    return $xml;
  }
  
  static public function getValidationRules() {
    $rules = array(
        "event_name"=>array("required"=>true),
        "event_title"=>array("required"=>true),
        "reg_email"=>array("email"=>true),
        "reg_start"=>array("date_".getenv("LC_ALL")=>true),
        "reg_end"=>array("date_".getenv("LC_ALL")=>true),
        "event_email"=>array("email"=>true)
    );
    $messages = array(
        "event_name"=>_("Please enter a short name for the event."),
        "event_title"=>_("Please enter the (descriptive) title for the event."),
        "reg_email"=>_("Please enter a valid email address (e.g. you@yourdomain.com)"),
        "reg_start"=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")),
        "reg_end"=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL")),
        "event_email"=>_("Please enter a valid email address (e.g. you@yourdomain.com)")
    );
    
    if(isset($_SESSION['event']) && $_SESSION['event']->event_id) {
      if (EventOptionsGroup::usingWaitlist($_SESSION['event']->event_id)) {
        $rules["enable_waitlist"] = array("required"=>true);
        $messages["enable_waitlist"] = _("Event options with enabled waitlist still exist.");
      }
    }
    
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }

  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    if(parent::saveForm($DB)) {
      if (isset($_SESSION['event']) && $_SESSION['event']->event_id==$this->data->event_id)
        $_SESSION['event']->readData($DB, array("event_id"=>$_SESSION['event']->event_id));
      return 1;
    }
    return 0;
  }  
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function getHTML(DbConnection $DB) {
?>
<form id="kdb_form" action="" method="post" enctype="multipart/form-data">
<div class="form_controls_top">
<p>
  <input type="submit" id="kdb_save" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('EventUI', 'save'); return false;">
  <input type="submit" id="kdb_save_as" name="kdb_save_as" value="<?=_("Save as new")?>" onClick="kdbSaveEntry('EventUI', 'saveasnew'); return false;">
  <input type="submit" id="kdb_delete" name="kdb_delete" value="<?=_("Delete")?>" onClick="kdbDeleteEntry('EventUI', 'kdb_form', '<?=_("Do you really want to delete this event? Attention: All booking data for this event will be permanently deleted as well.")?>'); return false;">
<?php
    if(isset($_SESSION['event']))
      echo "<input type=\"button\" id=\"continue_event_forms\" onClick=\"window.location.href='".KDbConfig::getCwfHttpPath()."admin/index.php/event_forms';\" class=\"continue\" value=\""._("Create registration from =>")."\">";
    else
      echo "<input disabled type=\"button\" id=\"continue_event_forms\" onClick=\"window.location.href='".KDbConfig::getCwfHttpPath()."admin/index.php/event_forms';\" class=\"continue\" value=\""._("Create registration from =>")."\">";
?>
</p>
</div>

<input type="hidden" id="event_id" name="event_id" class="id" value="<?= $this->data->event_id ?>">

<h3 class="first"><?=_("Event information")?></h3>

<p>
  <label for="event_name" class="required"><?=_("Short title")?>:</label>
  <input type="text" id="event_name" name="event_name" value="<?=$this->data->event_name?>" class="required">
    <span id="tt_event_name" class="tooltip"><?=_("The short title of the event is used in select fields. Otherwise the event title (see below) is displayed.")?></span>
</p>

<p>
  <label for="event_title" class="required"><?=_("Event title")?>:</label>
  <input type="text" id="event_title" name="event_title" class="required" value="<?=$this->data->event_title?>">
</p>

<p>
  <label for="event_location"><?=_("Location")?>:</label>
  <input type="text" id="event_location" name="event_location" value="<?=$this->data->event_location?>">
</p>

<p>
  <label for="event_period"><?=_("Period")?>:</label>
  <input type="text" id="event_period" name="event_period" value="<?=$this->data->event_period?>">
</p>

<p>
  <label for="event_email"><?=_("Email")?>:</label>
  <input type="text" id="event_email" name="event_email" value="<?=$this->data->event_email?>">
    <span id="tt_event_email" class="tooltip"><?=_("The email address is used as sender of emails (confirmation emails or emails send from views).")?></span>
</p>

<p>
  <label for="event_description"><?=_("Description")?>:</label>
  <textarea id="event_description" name="event_description"><?=$this->data->event_description?></textarea>
    <span id="tt_event_description" class="tooltip"><?=_("The description will be displayed in the event list.")?></span>
</p>

<h3><?=_("Event settings")?></h3>

<p>
  <label for="db_admin_group_id"><?=_("Admin group")?>:</label>
  <select id="db_admin_group_id" name="db_admin_group_id">
    <option value="-1"><?=_("Choose group")?></option>
<?php
    $groups = DbAdminGroup::readList($DB);
    foreach ($groups as $group)
      if($_SESSION['admin']['login']=="root" || in_array($group['id'], $_SESSION['admin']['db_admin_group_ids'])) {
        if($this->data->db_admin_group_id==$group['id'])
          echo "    <option value={$group['id']} selected>{$group['name']}</option>\n";
        else
          echo "    <option value={$group['id']}>{$group['name']}</option>\n";
      }
?>
  </select>
  <span id="tt_db_admin_group_id" class="tooltip"><?= _("Admin group that has access to this event. Members of that group can change the event, view registrations, etc.") ?></span>
</p>

<p>
  <label for="jump2url"><?=_("After registration go to URL")?>:</label>
  <input type="text" id="jump2url" name="jump2url" value="<?=$this->data->jump2url?>">
    <span id="tt_jump2url" class="tooltip"><?=_("After registration the user is automatically forwarded to this URL.")?></span>
</p>

<p>
  <label for="external_css"><?=_("External stylesheet (URL)")?>:</label>
  <input type="text" id="external_css" name="external_css" value="<?=$this->data->external_css?>">
    <span id="tt_external_css" class="tooltip"><?=_("External CSS stylesheet used to format the registration pages.")?></span>
</p>

<p>
  <label for="reg_email"><?=_("Email registration notifications to")?>:</label>
  <input type="text" id="reg_email" name="reg_email" value="<?=$this->data->reg_email?>">
    <span id="tt_reg_email" class="tooltip"><?=_("Send a notification to this email address, if a user registers for an event. If this is empty, no notifications will be sent.")?></span>
</p>

<h3><?=_("Registration settings")?></h3>

<p>
  <label for="disclaimer_id"><?=_("Show disclaimer")?>:</label>
  <select id="disclaimer_id" name="disclaimer_id">
    <option value="0"><?=_("no disclaimer")?></option>
<?php
    $disclaimers = Disclaimer::readList($DB);
    foreach ($disclaimers as $disclaimer)
      if($this->data->disclaimer_id==$disclaimer['id'])
        echo "    <option value={$disclaimer['id']} selected>{$disclaimer['name']}</option>\n";
      else
        echo "    <option value={$disclaimer['id']}>{$disclaimer['name']}</option>\n";
?>
  </select>
  <span id="tt_disclaimer_id" class="tooltip"><?= _("Show the selected disclaimer when a user is registering for this event.") ?></span>
</p>

<?php
    if(!KDbConfig::getEnableMultipleRegistration())
      echo "  <input type=\"hidden\" id=\"allow_multiple_reg\" name=\"allow_multiple_reg\" value=\"0\">\n";
    else {
      $label = _("Allow multiple registrations");
      $tooltip = _("Users may register multiple times for this event. This is especially useful for reoccuring events.");
      $value = (($this->data->allow_multiple_reg) ? " checked" : "");
      echo <<<EOD
<p>
  <label for="allow_multiple_reg">{$label}:</label>
  <input type="checkbox" id="allow_multiple_reg" name="allow_multiple_reg" value="1"{$value}>
    <span id="tt_allow_multiple_reg" class="tooltip">{$tooltip}</span>
</p>

EOD;
    }
?>
<p>
  <label for="enable_waitlist"><?=_("Enable waiting list")?>:</label>
  <input type="checkbox" id="enable_waitlist" name="enable_waitlist" value="1"<?=($this->data->enable_waitlist ? " checked" : "")?>>
    <span id="tt_enable_waitlist" class="tooltip"><?=_("Allow users to book booked out event options. They will be added to a waiting list for this event option.")." "._("The waitlist has to be enabled for individual option groups in the event form.")?></span>
</p>

<?php
    if(KDbConfig::getDisableFileUpload())
      echo "  <input type=\"hidden\" id=\"max_abstracts\" name=\"max_abstracts\" value=\"0\">\n";
    else {
      $label = _("Number of abstracts");
      $tooltip = _("The maximum number of abstracts the user is allowed to submit.");
      echo <<<EOD
<p>
  <label for="max_abstracts">{$label}:</label>
  <input type="text" id="max_abstracts" name="max_abstracts" value="{$this->data->max_abstracts}">
    <span id="tt_event_name" class="tooltip">{$tooltip}</span>
</p>
EOD;
    }
?>

<p>
  <label for="reg_start"><?=_("Online registration from")?>:</label>
  <input type="text" id="reg_start" name="reg_start" value="<?=CWFTools::dateISO2Local($this->data->reg_start, getenv("LC_ALL"))?>" style="width:100px;" placeholder="<?=CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))?>">
    <span id="tt_reg_start" class="tooltip"><?=_("Online registration is possible on and after the entered date.")?></span>
  <label for="reg_end" style="width:auto;"><?=_("to")?>:</label>
  <input type="text" id="reg_end" name="reg_end" value="<?=CWFTools::dateISO2Local($this->data->reg_end, getenv("LC_ALL"))?>" style="width:100px;" placeholder="<?=CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))?>">
    <span id="tt_reg_end" class="tooltip"><?=_("Online registrations ends on the entered date.")?></span>
</p>

<p>
  <label for="reg_start_txt"><?=_("Before registration period")?>:</label>
  <input type="text" id="reg_start_txt" name="reg_start_txt" value="<?=$this->data->reg_start_txt?>">
    <span id="tt_reg_start_txt" class="tooltip"><?=_("Display this text before online registration is possible.")?></span>
</p>

<p>
  <label for="reg_end_txt"><?=_("After registration period")?>:</label>
  <input type="text" id="reg_end_txt" name="reg_end_txt" value="<?=$this->data->reg_end_txt?>">
    <span id="tt_reg_end_txt" class="tooltip"><?=_("Display this text after online registration period has ended.")?></span>
</p>

<p>
  <label for="confirmation_text"><?=_("Confirmation text")?>:</label>
  <textarea id="confirmation_text" name="confirmation_text"><?=$this->data->confirmation_text?></textarea><br>
  <span class="form_additional_info"><b><?=_("Variables")?>:</b> $FORM_DATA$ (<?=_("prints the data entered in the registration form")?>), $salutation$, $givenname$, $familyname$, $user_id$, $passwd$, $booked_program$, $grandtotal$</span>
    <span id="tt_confirmation_text" class="tooltip"><?=_("This text is displayed after registration. The optional variables are replaced with the data entered by the user.")?></span>
</p>

<p>
  <label for="confirmation_email"><?=_("Text for confirmation email")?>:</label>
  <textarea id="confirmation_email" name="confirmation_email"><?=$this->data->confirmation_email?></textarea><br>
  <span class="form_additional_info"><b><?=_("Variables")?>:</b> $FORM_DATA$ (<?=_("prints the data entered in the registration form")?>), $salutation$, $givenname$, $familyname$, $user_id$, $passwd$, $booked_program$, $grandtotal$</span>
    <span id="tt_confirmation_email" class="tooltip"><?=_("This text is used in the confirmation email after registration. The optional variables are replaced with the data entered by the user.")?></span>
</p>

<div id="confirmation_waitlist"<?=(($this->data->enable_waitlist) ? "" : ' style="display:none;"')?>>
<p>
  <label for="confirmation_text_wl"><?=_("Confirmation text (waitlist)")?>:</label>
  <textarea id="confirmation_text_wl" name="confirmation_text_wl"><?=$this->data->confirmation_text_wl?></textarea><br>
  <span class="form_additional_info"><b><?=_("Variables")?>:</b> $FORM_DATA$ (<?=_("prints the data entered in the registration form")?>), $salutation$, $givenname$, $familyname$, $user_id$, $passwd$, $booked_program$, $grandtotal$</span>
    <span id="tt_confirmation_text_wl" class="tooltip"><?=_("This text is displayed after registration, if the user was waitlisted.")?></span>
</p>

<p>
  <label for="confirmation_email_wl"><?=_("Text for confirmation email (waitlist)")?>:</label>
  <textarea id="confirmation_email_wl" name="confirmation_email_wl"><?=$this->data->confirmation_email_wl?></textarea><br>
  <span class="form_additional_info"><b><?=_("Variables")?>:</b> $FORM_DATA$ (<?=_("prints the data entered in the registration form")?>), $salutation$, $givenname$, $familyname$, $user_id$, $passwd$, $booked_program$, $grandtotal$</span>
    <span id="tt_confirmation_email_wl" class="tooltip"><?=_("This text is used in the confirmation email, if the user was waitlisted.")?></span>
</p>
</div>

<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('EventUI', 'save'); return false;">
  <input type="submit" id="kdb_save_as2" name="kdb_save_as" value="<?=_("Save as new")?>" onClick="kdbSaveEntry('EventUI', 'saveasnew'); return false;">
  <input type="submit" id="kdb_delete2" name="kdb_delete" value="<?=_("Delete")?>" onClick="kdbDeleteEntry('EventUI', 'kdb_form', '<?=_("Do you really want to delete this event? Attention: All booking data for this event will be permanently deleted as well.")?>'); return false;">
<?php
    if(isset($_SESSION['event']))
      echo "<input type=\"button\" onClick=\"window.location.href='".KDbConfig::getCwfHttpPath()."admin/index.php/event_forms';\" class=\"continue\" value=\""._("Create registration from =>")."\">";
?>
</p>
</div>
</form>
<?php
  }
}

?>