<?php 
require_once("icwfData.inc.php");
require_once("FormValidation.inc.php");

class KDbOptions implements icwfData {
  private static $noExport = array();
  private static $notInArray = array("validation");
  private $validation;
  
  private $organisation_name;
  private $root_email;
  private $allow_user_login;
  private $disable_file_upload;
  private $enable_multiple_registration;
  private $email_duplicates;
  private $css_skin;
  private $force_language;
  private $db_host;
  private $db_name;
  private $db_user;
  private $db_passwd;
  private $admin_login_mode;
  private $currency;
  private $user_properties;
  private $user_properties_req;
  private $confirmation_text;
  private $confirmation_email;
  private $returnpath_workaround;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  private function initialize() {
    foreach ($this as $key => $value)
      if(in_array($key, array("user_properties", "user_properties_req")))
        $this->$key = array();
      else
        $this->$key = null;
  }
  
  public function __construct(FormValidation $validation=null) {
    $this->initialize();
    
    if(!defined("THIS_FILE") || THIS_FILE!="setup.php")
      $this->readData();
    
    $this->validation = $validation;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // get values
  public function __get($name) {
    if (array_key_exists($name, get_class_vars(get_class())) && !in_array($name, self::$noExport))
      return $this->$name;
    return null;
  }

  public function getIDName() {
    return "";
  }
  
  public function toArray() {
    $array = array();
    foreach ($this as $key => $value)
      if(!in_array($key, self::$notInArray))
        $array[$key] = $value; 
    return $array;
  }
  
  public function getPostBackData() {
    return array();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////////
  // set values
  public function setValidation(FormValidation $validation) {
    $this->validation = $validation;
  }
  
  private function checkConnection() {
    $checkDb = new DbConnection($this->db_host, $this->db_user, $this->db_passwd, $this->db_name);
    if ($checkDb->getConnectionError()) {
      $this->validation->addInvalidField("db_connection", _("Database connection failed."));
      return 0;
    }

    return 1;
  }
  
  public function fromArray($array) {
    foreach ($this as $key => $value) {
      switch ($key) {
        case "validation":
          break;
          
        case "allow_user_login":
        case "disable_file_upload":
        case "enable_multiple_registration":
        case "returnpath_workaround":
          $this->$key = ((isset($array[$key]) && $array[$key]) ? 1 : 0);
          break;
        case "email_duplicates":
          if(isset($array[$key]) && 
             in_array($array[$key], array(EMAIL_DONT_CHECK, EMAIL_CHECK_EVENT, EMAIL_ALWAYS_CHECK)))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
        case "admin_login_mode":
          if(isset($array[$key]) && 
             in_array($array[$key], array(LOGIN_MODE_IP, LOGIN_MODE_PASSWD, LOGIN_MODE_IP_OR_PW, LOGIN_MODE_IP_AND_PW)))
            $this->$key = $array[$key];
          else
            $this->$key = null;
          break;
          
        default:
          if(isset($array[$key]))
            $this->$key = $array[$key];
          else if(in_array($key, array("user_properties", "user_properties_req")))
            $this->$key = array();
          else
            $this->$key = null;
          break;
      }
    }
    
    $array['db_connection'] = 1;
    return ($this->checkConnection() && $this->validation->validate($array));
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - read
  public function readData(DbConnection $DB=null, $conditions="") {
    $this->organisation_name = KDbConfig::getOrganisationName();
    $this->root_email = KDbConfig::getRootEmail();
    $this->allow_user_login = KDbConfig::getAllowUserLogin();
    $this->disable_file_upload = KDbConfig::getDisableFileUpload();
    $this->enable_multiple_registration = KDbConfig::getEnableMultipleRegistration();
    $this->email_duplicates = KDbConfig::getEmailDuplicates();
    $this->css_skin = KDbConfig::getCssSkin();
    $this->force_language = (KDbConfig::getUseLang() ? "" : KDbConfig::getDefaultLang());
    $this->db_host = KDbConfig::getDbHost();
    $this->db_name = KDbConfig::getDbName();
    $this->db_user = KDbConfig::getDbUser();
    $this->db_passwd = KDbConfig::getDbPasswd();
    $this->admin_login_mode = KDbConfig::getAdminLoginMode();
    $this->currency = KDbConfig::getCurrency();
    $this->user_properties = explode(",", KDbConfig::getUserProperties());
    $this->user_properties_req = explode(",", KDbConfig::getUserPropertiesReq());
    $this->confirmation_text = KDbConfig::getKdbConfirmationText();
    $this->confirmation_email = KDbConfig::getKdbConfirmationEmail();
    $this->returnpath_workaround = KDbConfig::getReturnpathWorkaround();
    return array(0, "");
  }
  
  static public function readList(DbConnection $DB) {
    return array();
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // database operations - write
  public function writeData(DbConnection $DB) {
    $useLang = ($this->force_language=="" ? 1 : 0);
    $forceLang = ($this->force_language=="" ? "de" : $this->force_language);
    $cwf_path = rtrim(dirname(dirname($_SERVER['SCRIPT_FILENAME'])), '/').'/';
    $cwf_http_path = rtrim(dirname(dirname(preg_replace('~\.php/.*~', '.php', $_SERVER['PHP_SELF']))), '/').'/';
    $user_properties = implode(",", $this->user_properties);
    $user_properties_req = implode(",", $this->user_properties_req);
    
    $configFile = $cwf_path."config/config.inc.php";
    if(file_exists($configFile) && !is_writable($configFile))
      return array(1, _("Config file '{$configFile}' is readonly."));
    
    $data = <<<EOD
<?php
require_once("{$cwf_path}k_db/KDbConfig.inc.php");

// CWF options
KDbConfig::setCwfPath('{$cwf_path}');
KDbConfig::setCwfHttpPath('{$cwf_http_path}'); 
ini_set("include_path", KDbConfig::getCwfPath()."cwf".PATH_SEPARATOR.KDbConfig::getCwfPath()."k_db".PATH_SEPARATOR.KDbConfig::getCwfPath()."admin");
KDbConfig::setAutoloaderSrc('cwf/'.PATH_SEPARATOR.'k_db/'.PATH_SEPARATOR.'admin/');
KDbConfig::setUseLang({$useLang});
KDbConfig::setDefaultLang('{$forceLang}');

KDbConfig::setOrganisationName('{$this->organisation_name}');
KDbConfig::setRootEmail('{$this->root_email}');
KDbConfig::setAllowUserLogin('{$this->allow_user_login}');
KDbConfig::setDisableFileUpload('{$this->disable_file_upload}');
KDbConfig::setEnableMultipleRegistration('{$this->enable_multiple_registration}');
KDbConfig::setEmailDuplicates('{$this->email_duplicates}');
KDbConfig::setCssSkin('{$this->css_skin}');
KDbConfig::setCurrency('{$this->currency}');

KDbConfig::setReturnpathWorkaround('{$this->returnpath_workaround}');

//Database settings
KDbConfig::setDbHost('{$this->db_host}');
KDbConfig::setDbName('{$this->db_name}');
KDbConfig::setDbUser('{$this->db_user}');
KDbConfig::setDbPasswd('{$this->db_passwd}');

//Security settings
KDbConfig::setAdminLoginMode('{$this->admin_login_mode}');

//User properties
KDbConfig::setUserProperties('{$user_properties}');
KDbConfig::setUserPropertiesReq('{$user_properties_req}');

//Registration confirmation
KDbConfig::setKdbConfirmationText('{$this->confirmation_text}');
KDbConfig::setKdbConfirmationEmail('{$this->confirmation_email}');
?>
EOD;

    file_put_contents($configFile, $data);
    return array(0, "");
  }

  static public function deleteData(DbConnection $DB, $id) {
    return array(0, "");
  }
}
?>