<?php
class CWFMainUI {
  /*
   * Format 
   */
  protected $MENU;
  protected $language;
  protected $defaultLang;
  protected static $allowed_languages = array("de", "en");
  protected $get_vars;
  protected $absolutePath;
  
  protected $errorMsg;
  protected $fatalError;
  protected $displayPage;
  protected $displayMenu;

  /* ****************************************************************************************
   * Constructor
   */
  public function __construct($displayMenu=true) {
    $this->errorMsg = "";
    $this->displayPage = true;
    $this->displayMenu = $displayMenu;
    
    unset($_GET['menu_select']);
    //$this->add_getvars();
    
    if (!KDbConfig::getDefaultLang() || !in_array(KDbConfig::getDefaultLang(), static::$allowed_languages))
      KDbConfig::setDefaultLang("de");
    $this->defaultLang = KDbConfig::getDefaultLang();

    if (KDbConfig::getUseLang()) {
      if (isset($_GET['lang']) && preg_match('~^[a-z]{2}$~', $_GET['lang']) && in_array($_GET['lang'], static::$allowed_languages))
        $this->language = $this->lang_getfrombrowser($_GET['lang']);
      else {
        unset($_GET['lang']);
        $this->language = $this->lang_getfrombrowser();
      }
    }
    else {
      unset($_GET['lang']);
      $this->language = KDbConfig::getDefaultLang();
    }

    $locale = $this->getLanguage();
    putenv("LC_ALL=$locale");
    setlocale(LC_ALL, $locale);
    bindtextdomain("default", KDbConfig::getCwfPath()."locale");
    textdomain("default");
    bind_textdomain_codeset("default", "UTF-8");
    
    $this->absolutePath = "";

    if($this->displayMenu)
      $this->MENU = $this->createMenu();
  }
  
  /* ****************************************************************************************
   * Get language
   */
  final public function getLanguage() {
    switch($this->language) {
      case 'de':
        return "de_DE";
      case 'en':
        return "en_US";
    }
  }
  
  final static public function allowedLanguages() {
    return static::$allowed_languages;
  }
  
  /* ****************************************************************************************
   * Set/Get error message
   */
  final public function setErrorMessage($error_msg, $fatalError=0) {
    if ($error_msg)
      $this->errorMsg .= "<p>{$error_msg}</p>\n";
    
    $this->fatalError = (isset($fatalError) && $fatalError);
  }
  final public function getErrorMessage() {
    return $this->errorMsg;
  }
  
  /* ****************************************************************************************
   * Print page header
   */
  public function writeHeader($currentMenu, $title, $addHeader) {
    global $DB;
?>
<!doctype html>
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <link rel="stylesheet" href="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/css/cwfForms.css" type="text/css">
  <link rel="stylesheet" href="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/css/cwfMenu.css" type="text/css">
  <link rel="stylesheet" href="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/css/cwfAjax.css" type="text/css">
  <script type="text/javascript">var CWF_HTTP_Path="<?php echo KDbConfig::getCwfHttpPath(); ?>";</script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/jquery-1.11.0.min.js" type="text/javascript"></script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/jquery.hoverIntent.js" type="text/javascript"></script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/jquery.validate.min.js" type="text/javascript"></script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/jquery.form.js" type="text/javascript"></script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/cwfCommon.js" type="text/javascript"></script>
  <script src="<?php echo KDbConfig::getCwfHttpPath(); ?>cwf/js/validation_rules.js" type="text/javascript"></script>
<?php
    echo $addHeader;
    echo "  <title>".(($title) ? "{$title}" : "")."</title>\n"; 
?>
</head>
<body>

<div id="logo"></div>

<div id="ajax_message">
  <div id="ajax_message_bg"></div>
  <div id="ajax_message_container"></div>
</div>

<?php
    echo "<div id=\"menu\">\n";
    if($this->displayMenu && !$this->fatalError) {
      echo "  <ul id=\"dropdownmenu\">\n";
      echo $this->getMenu($currentMenu, $this->MENU);
      echo "  </ul>\n";
    }
    echo "</div>\n";
?>
<div id="content">

<?php
    if($this->fatalError)
      $this->displayPage = false;
    
    if ($this->errorMsg == "")
      echo "<div id=\"error_message\"></div>\n";
    else
      echo "<div id=\"error_message\" style=\"display:block;\">\n{$this->errorMsg}</div>\n";
    
    return $this->displayPage;
  }

  /* ****************************************************************************************
   * Print page footer
   */
  public function writeFooter() {
?>
</div>
</body>
</html>
<?php    
  }
  
  /* ****************************************************************************************
   * Check if permission for a menu point is granted
   */
  protected function checkPermission ($permission) {
    switch ($permission['type']) {
      case 'isset':
        $perm = (eval("return (isset({$permission['rule']}) && {$permission['rule']});"));
        break;
      case 'user':
        $perm = ($_SESSION['admin']['login']==$permission['rule'] || $_SESSION['admin']['login']=="root");
        break;
      case 'perm':
        $perm = (in_array($permission['rule'], $_SESSION['admin']['access_rights']) || $_SESSION['admin']['login']=="root");
        break;
    }
    
    if(isset($permission['not']) && $permission['not'])
      $perm = (!$perm);
    
    return $perm;
  }
  
  protected function hasPermission($permission, &$message="") {
    $message = "";
    if(!is_array($permission)) return true;

    if(!isset($permission['type'])) {
      foreach($permission as $perm) {
        if(!$this->checkPermission($perm)) {
          $message = (isset($perm['message']) ? $perm['message'] : "");
          return false;
        }
      }
      return true;
    }
    else {
      if (!$hasPermission=$this->checkPermission($permission))
        $message = (isset($permission['message']) ? $permission['message'] : "");
      return $hasPermission;
    }
  }
  
  /* ****************************************************************************************
   * Create menu array
   */
  protected function createMenu() {
    /*
     * title:      String
     * url:        String (starting with "!": link is disabled, use rest of url as title attribute text
     * permission: array('type'=>(user|perm), 'rule'=>String
     * submenu:    array of menupoints 
     */
    return array(
      'MP1'=>array('title'=>'',  'url'=>'',            'permission'=>'', 'submenu'=>'')
    );
  }
  
  public function addMenuItem($newMenuItem, $afterItem) {
    if(!$afterItem || count($afterItem)==0)
      $this->MENU = array_merge($newMenuItem, $this->MENU);
    else if(is_array($afterItem)) {
      $currentSubMenu = &$this->MENU;
      $afterMenuItem = array_pop($afterItem);
      foreach ($afterItem as $subMenu) {
        $currentSubMenu = &$currentSubMenu[$subMenu]['submenu'];
      }
      if($afterMenuItem=="")
        $currentSubMenu = array_merge($newMenuItem, $currentSubMenu);
      else {
        $newSubMenu = array();
        foreach($currentSubMenu as $key => $value) {
          $newSubMenu[$key] = $value;
          if($afterMenuItem==$key)
            $newSubMenu = array_merge($newSubMenu, $newMenuItem);
        }
        $currentSubMenu = $newSubMenu;
      }
    }
  }
  
  /* ****************************************************************************************
   * Create menu
   */
  final protected function getMenu($current, $menu, $indent="  ", $level=1) {
    $str = "";
    $cache = "";
    foreach ($menu as $idx => $menupoint) {
      $errorMessage = "";
      if(isset($menupoint['permission']) && !$this->hasPermission($menupoint['permission'], $errorMessage)) {
        if ($menupoint['url']==$current) {
          $this->displayPage = false;
          $this->setErrorMessage(_("You don't have permission to view this page."));
        }
        if (!$errorMessage)
          continue;
      }
      if ($menupoint['title']=="") {
        if($str != "")  $cache = $indent."  <li class=\"menuitem hr\"></li>\n";
      }
      else if (preg_match('~^javascript:~', $menupoint['url'])) {
        if($errorMessage)
          $str .= $cache . $indent."  <li class=\"menuitem disabled\"><span class=\"menutext\" style=\"cursor:pointer\">{$menupoint['title']}</span></li>\n";
        else
          $str .= $cache . $indent."  <li class=\"menuitem\"><a class=\"menutext\" style=\"cursor:pointer\" href=\"{$menupoint['url']}\">{$menupoint['title']}</a></li>\n";
        $cache = "";
      }
      else if (preg_match('~^https?://~', $menupoint['url']))
        $str .= $indent."  <li class=\"menuitem external\"><a class=\"menutext\" href=\"{$menupoint['url']}\">{$menupoint['title']}</a></li>\n";
      else {
        $classes = "menuitem";
        if ($menupoint['url']==$current)
          $classes .= " selected";
        if(substr($menupoint['url'], 0, 1)=="!" || $errorMessage)
          $classes .= " disabled";

        $hint = "";
        if(is_array($menupoint['submenu'])) {
          $classes .= " submenu";
          $hint = "<span class=\"submenuhint\">&raquo;</span>";
        }
        
        if ($errorMessage)
          $link = "<span class=\"menutext\" id=\"menutext_$idx\" name=\"menutext_$idx\">{$menupoint['title']}</span><span class=\"tooltip\" id=\"tt_menutext_$idx\">{$errorMessage}</span>";
        else if (!$menupoint['url'] || substr($menupoint['url'], 0, 1)=="!") {
          $link = "<a class=\"menutext\" href=\"#\">{$menupoint['title']}</a>";
        }
        else {
          $link = "<a class=\"menutext\" href=\"{$this->absolutePath}{$menupoint['url']}{$this->get_vars}\">{$menupoint['title']}</a>";
        }

        if(is_array($menupoint['submenu']) && !$errorMessage) {
          $str_submenu = "";
          if (count($menupoint['submenu'])>0)
            $str_submenu = $this->getMenu($current, $menupoint['submenu'], $indent."    ", $level+1);
          if ($str_submenu!="") {
            $strSub = "\n";
            $strSub .= $indent."    <ul>\n";
            $strSub .= $str_submenu;
            $strSub .= $indent."    </ul>\n";
          }
          else
            $strSub = "";
          
          if($menupoint['url'] || $strSub!="") {
            $str .= $cache . $indent."  <li id=\"menuitem_{$idx}\" class=\"$classes\">{$link}{$hint}";
            $str .= $strSub;
            $str .= $indent."  </li>\n";
            $cache = "";
          }
        }
        else if(is_string($menupoint['submenu']) && $menupoint['submenu']!="" && !$errorMessage) {
          $str .= $cache . $indent."  <li id=\"menuitem_{$idx}\" class=\"$classes\">{$link}{$hint}\n";
          $callback = $menupoint['submenu'];
          $str .= $this->$callback();
          $str .= $indent."  </li>\n";
          $cache = "";
        }
        else {
          $str .= $cache . $indent."  <li id=\"menuitem_{$idx}\" class=\"$classes\">{$link}</li>\n";
          $cache = "";
        }
      }
    }
    
    return $str;
  }

  public function getMenuPublic($current, $menu) {
    return $this->getMenu($current, $menu);
  }
  
  /* ****************************************************************************************
   * Add $_GET variables to URL
   */
  final protected function add_getvars() {
    $get_vars = array();
    
    if (count($_GET)>0) {
      foreach($_GET as $key => $value)
        $get_vars[$key] = "$key=$value";
    }
  
    if (count($get_vars)>0)
      $this->get_vars = "?".implode('&', $get_vars);
    else
      $this->get_vars = "";
  }
  
  /* ****************************************************************************************
   * Get language from browser
   */
  final protected function lang_getfrombrowser ($lang_variable = null, $strict_mode = false) {
    // $_SERVER['HTTP_ACCEPT_LANGUAGE'] verwenden, wenn keine Sprachvariable mitgegeben wurde
    if ($lang_variable === null && isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
      $lang_variable = $_SERVER['HTTP_ACCEPT_LANGUAGE'];
    }
    
    // wurde irgendwelche Information mitgeschickt?
    if (empty($lang_variable)) {
      // Nein? => Standardsprache zurückgeben
      return $this->defaultLang;
    }
    
    // Den Header auftrennen
    $accepted_languages = preg_split('/,\s*/', $lang_variable);
    
    // Die Standardwerte einstellen
    $current_lang = $this->defaultLang;
    $current_q = 0;
    
    // Nun alle mitgegebenen Sprachen abarbeiten
    foreach ($accepted_languages as $accepted_language) {
      // Alle Infos über diese Sprache rausholen
      $res = preg_match ('/^([a-z]{1,8}(?:-[a-z]{1,8})*)'.
                         '(?:;\s*q=(0(?:\.[0-9]{1,3})?|1(?:\.0{1,3})?))?$/i', $accepted_language, $matches);
    
      // war die Syntax gültig?
      if (!$res) {
        // Nein? Dann ignorieren
        continue;
      }
      
      // Sprachcode holen und dann sofort in die Einzelteile trennen
      $lang_code = explode ('-', $matches[1]);
    
      // Wurde eine Qualität mitgegeben?
      if (isset($matches[2])) {
        // die Qualität benutzen
        $lang_quality = (float)$matches[2];
      } else {
        // Kompabilitätsmodus: Qualität 1 annehmen
        $lang_quality = 1.0;
      }
    
      // Bis der Sprachcode leer ist...
      while (count ($lang_code)) {
        // mal sehen, ob der Sprachcode angeboten wird
        if (in_array (strtolower (join ('-', $lang_code)), static::$allowed_languages)) {
          // Qualität anschauen
          if ($lang_quality > $current_q) {
            // diese Sprache verwenden
            $current_lang = strtolower (join ('-', $lang_code));
            $current_q = $lang_quality;
            // Hier die innere while-Schleife verlassen
            break;
          }
        }
        // Wenn wir im strengen Modus sind, die Sprache nicht versuchen zu minimalisieren
        if ($strict_mode) {
          // innere While-Schleife aufbrechen
          break;
        }
        // den rechtesten Teil des Sprachcodes abschneiden
        array_pop ($lang_code);
      }
    }
    
    // die gefundene Sprache zurückgeben
    return $current_lang;
  }
}
?>
