<?php
require_once("cwfUI.inc.php");

class UserUI extends cwfUI {
  protected static $accessRight = "";
  protected static $dataClass = "User";
  public $data;
  private $DB;
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // initialize
  public function __construct(DbConnection $DB) {
    parent::__construct();
    $this->DB = $DB;
    $this->initialize();
    
    if(isset($_POST['kdb_save']) && $_POST['kdb_save']) {
      $this->saveForm($DB);
    }
  }
  
  protected function initialize() {
    parent::initialize();
    if (isset($_SESSION['user'])) {
      $this->data = clone $_SESSION['user'];
      $this->data->setValidation(self::getValidationRules());
    }
    else if(isset($_SESSION['registration']['user_id']) && $_SESSION['registration']['user_id']) {
      $this->data = new static::$dataClass();
      $this->data->readData($this->DB, "user_id='{$_SESSION['registration']['user_id']}'");
      $this->data->setValidation(self::getValidationRules());
    }
    else
      $this->data = new static::$dataClass(self::getValidationRules());
    
    $this->title = _("Registration");
    $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
    /*
    $this->header = "  <script type=\"text/javascript\">
      $(document).ready(function(){
        jQuery.validator.messages.required = '';
        $(\"form\").validate({
          invalidHandler: function(e, validator) {
            var errors = validator.numberOfInvalids();
            if (errors) {
              var message = errors == 1
              ? "._("'You missed 1 field. It has been highlighted below.'")."
              : "._("'You missed ' + errors + ' fields. They have been highlighted below.'").";
              $(\"div#error_message\").html(message);
              $(\"div#error_message\").show();
            } else {
              $(\"div#error_message\").hide();
            }
          },".
          $this->data->validation->getValidationRulesJQuery("        ")."
        });
      });
    </script>\n";
    */
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // handle data
  static public function getValidationRules() {
    $userProperties = explode(",", KDbConfig::getUserProperties());
    $userPropertiesReq = explode(",", KDbConfig::getUserPropertiesReq());
    
    $addRules = array("email"=>array("email"=>true, "remote"=>array("url"=>KDbConfig::getCwfHttpPath()."admin/checkValue.php", "data"=>array("class"=>"User", "value"=>"#email", "id"=>"#user_id"))), 
                      "date_of_birth"=>array("date_".getenv("LC_ALL")=>true));
    $addMessages = array("email"=>array("email"=>_("Please enter a valid email address (e.g. you@yourdomain.com)"),
                                        "remote"=>_("Email address is already taken. Please enter a different address.")),
                         "date_of_birth"=>array("date_".getenv("LC_ALL")=>_("Please enter a valid date, e.g. ").CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))));
    $rules = array();
    $messages = array();
    foreach($userProperties as $property) {
      if(isset($addRules[$property]) && in_array($property, $userPropertiesReq)) {
        $rules[$property] = array_merge(array("required"=>true), $addRules[$property]);
        $messages[$property] = array_merge(array("required"=>_("This field is mandatory.")), $addMessages[$property]);
      }
      else {
        if(isset($addRules[$property])) {
          $rules[$property] = $addRules[$property];
          $messages[$property] = $addMessages[$property];
        }
        else if (in_array($property, $userPropertiesReq)) {
          $rules[$property] = array("required"=>true);
          $messages[$property] = array("required"=>_("This field is mandatory."));
        }
      }
    }
    
    $validationRules = new FormValidation($rules, $messages);
    return $validationRules;
  }
  
  public function addValidationRules(FormValidation $validationRules) {
    $this->data->validation->addRules($validationRules);

    $this->header = "  <script type=\"text/javascript\">
    $(document).ready(function(){
      $(\"form\").validate({
        errorClass: 'invalid',\n".
        $this->data->validation->getValidationRulesJQuery("        ")."
      });
    });
  </script>\n";
  }
  
  public function getPostBackXML(DbConnection $DB, $action) {
    $xml = parent::getPostBackXML($DB, $action);
    switch ($action) {
      case "save":
        if(isset($_SESSION['user'])) {
          $xml = $this->getNewValuesXML($this->data->getPostBackData());
        }
        break;
    }
    return $xml;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // Database operations
  public function saveForm(DbConnection $DB) {
    if($this->data->fromArray($_POST)) {
      list($errno, $error) = $this->data->writeData($DB);
      if ($errno) {
        $this->error = $error;
        return 0;
      }
      else if (isset($_SESSION['user']))
        $_SESSION['user'] = clone $this->data;
      return 1;
    }
    return 0;
  }
  
  /////////////////////////////////////////////////////////////////////////////////////////////////
  // output
  public function printSubmitButtons() {
    return 1;
  }
  
  protected function getHTMLuser_id($classes=array()) {
    return "<input type=\"hidden\" name=\"user_id\" id=\"user_id\" value=\"{$this->data->user_id}\">";
  }
  protected function getHTMLgender($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"sex\">"._("Gender").":</label>
  <input type=\"radio\" name=\"sex\" id=\"sex_m\" value=\"m\"".(($this->data->sex=="m") ? " checked" : "")."\"> <label for=\"sex_m\" class=\"for_radio\">male</label>&nbsp;&nbsp;&nbsp;
  <input type=\"radio\" name=\"sex\" id=\"sex_f\" value=\"f\"".(($this->data->sex=="f") ? " checked" : "")."\"> <label for=\"sex_f\" class=\"for_radio\">female</label>
</p>";
  }
  protected function getHTMLsalutation_id($classes=array()) {
    $classes[] = "select_id";
    $str = "<p>
  <label {$this->getClassString($classes)}for=\"salutation_id\">"._("Salutation").":</label>
  <select {$this->getClassString($classes)}id=\"salutation_id\" name=\"salutation_id\" size=\"1\">
    <option value=\"\">"._("Please select salutation")."</option>\n";
    $salutations = Salutation::readList($this->DB);
    foreach ($salutations as $salutation)
      if($salutation['id']==$this->data->salutation_id)
        $str .= "    <option value=\"{$salutation['id']}\" selected>{$salutation['name']}</option>\n";
      else
        $str .= "    <option value=\"{$salutation['id']}\">{$salutation['name']}</option>\n";
    $str .= "  </select>
</p>";
    return $str;
  }
  protected function getHTMLfamilyname($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"familyname\">"._("Family name").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"familyname\" id=\"familyname\" value=\"{$this->data->familyname}\">
</p>";
  }
  protected function getHTMLgivenname($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"givenname\">"._("Given name").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"givenname\" id=\"givenname\" value=\"{$this->data->givenname}\">
</p>";
  }
  protected function getHTMLcompany($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"company\">"._("Company/Institute").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"company\" id=\"company\" value=\"{$this->data->company}\">
</p>";
  }
  protected function getHTMLstreet($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"street\">"._("Street/P.O.Box").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"street\" id=\"street\" value=\"{$this->data->street}\">
</p>";
  }
  protected function getHTMLzip_code($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"zip_code\">"._("ZIP code").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"zip_code\" id=\"zip_code\" value=\"{$this->data->zip_code}\">
</p>";
  }
  protected function getHTMLcity($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"city\">"._("City").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"city\" id=\"city\" value=\"{$this->data->city}\">
</p>";
  }
  protected function getHTMLcountry_id($classes=array()) {
    $classes[] = "select_id";
    $str = "<p>
  <label {$this->getClassString($classes)}for=\"country_id\">"._("Country").":</label>
  <select {$this->getClassString($classes)}id=\"country_id\" name=\"country_id\" size=\"1\">
    <option value=\"\">"._("Please select country")."</option>\n";
    $countries = Country::readList($this->DB);
    foreach ($countries as $country)
      if($country['id']==$this->data->country_id)
        $str .= "    <option value=\"{$country['id']}\" selected>{$country['name']}</option>\n";
      else
        $str .= "    <option value=\"{$country['id']}\">{$country['name']}</option>\n";
    $str .= "  </select>
</p>";
    return $str;
  }
  protected function getHTMLphone($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"phone\">"._("Phone").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"phone\" id=\"phone\" value=\"{$this->data->phone}\">
</p>";
  }
  protected function getHTMLfax($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"fax\">"._("Fax").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"fax\" id=\"fax\" value=\"{$this->data->fax}\">
</p>";
  }
  protected function getHTMLemail($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"email\">"._("Email").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"email\" id=\"email\" value=\"{$this->data->email}\">
</p>";
  }
  protected function getHTMLdate_of_birth($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"date_of_birth\">"._("Date of birth").":</label>
  <input {$this->getClassString($classes)}type=\"text\" name=\"date_of_birth\" id=\"date_of_birth\" value=\"".CWFTools::dateISO2Local($this->data->date_of_birth, getenv("LC_ALL"))."\" placeholder=\"".CWFTools::dateISO2Local("2000-12-31", getenv("LC_ALL"))."\">
</p>";
  }
  protected function getHTMLdiseases_allergies($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"diseases_allergies\">"._("Diseases / allergies").":</label>
  <input type=\"text\" name=\"diseases_allergies\" id=\"diseases_allergies\" value=\"{$this->data->diseases_allergies}\">
</p>";
  }
  protected function getHTMLvegetarian($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"vegetarian\">"._("I am a vegetarian").":</label>
  <input type=\"checkbox\" name=\"vegetarian\" id=\"vegetarian\" value=\"1\"".(($this->data->vegetarian) ? " checked" : "").">
</p>";
  }
  protected function getHTMLparamedic($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"paramedic\">"._("I am a paramedic").":</label>
  <input type=\"checkbox\" name=\"paramedic\" id=\"paramedic\" value=\"1\"".(($this->data->paramedic) ? " checked" : "").">
</p>";
  }
  protected function getHTMLpublish_on_website($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"publish_on_website\">"._("Publish my name and affiliation on the website").":</label>
  <input type=\"checkbox\" name=\"publish_on_website\" id=\"publish_on_website\" value=\"1\"".(($this->data->publish_on_website) ? " checked" : "").">
</p>";
  }
  protected function getHTMLallow_email($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"allow_email\">"._("I want receive emails regarding future events").":</label>
  <input type=\"checkbox\" name=\"allow_email\" id=\"allow_email\" value=\"1\"".(($this->data->allow_email) ? " checked" : "").">
</p>";
  }
  protected function getHTMLcv_filename($classes=array()) {
    return "<p>
  <label {$this->getClassString($classes)}for=\"cv_filename\">"._("Curriculum vitae").":</label>
  <input type=\"file\" name=\"cv_filename\" id=\"cv_filename\">
</p>";
  }
  
  public function getHTMLView(DbConnection $DB) {
?>
<form id="kdb_form_user" action="" method="post" enctype="multipart/form-data">
<h3 class="first"><?=_("Registration data")?></h3>

<?php
    echo $this->getHTMLuser_id()."\n";

    $this->getHTMLRegistration($DB);
?>
<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_user', refreshHiddenColumns); return false;">
  <input type="submit" id="kdb_save_close2" name="kdb_save_close" value="<?=_("Save and close")?>"  onClick="kdbSaveEntry('ViewDataUI', 'save', 'kdb_form_user', refreshHiddenColumns); $('#dialog_view').hide(); return false;">
</p>
</div>
</form>
<?php
  }

  public function getHTMLAddReg(DbConnection $DB) {
?>
<form id="kdb_form_user" action="" method="post" enctype="multipart/form-data">
<h3 class="first"><?=_("Registration data")?></h3>

<?php
    echo $this->getHTMLuser_id()."\n";

    $this->getHTMLRegistration($DB);
?>
<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbAddRegNewUser('AddRegistrationUI', 'save', 'kdb_form_user'); return false;">
</p>
</div>
</form>
<?php
  }

  public function getHTML(DbConnection $DB) {
?>
<form id="kdb_form_user" action="" method="post" enctype="multipart/form-data">
<h3 class="first"><?=_("Registration data")?></h3>

<?php
    echo $this->getHTMLuser_id()."\n";

    $this->getHTMLRegistration($DB);
?>
<div class="form_controls_bottom">
<p>
  <input type="submit" id="kdb_save2" name="kdb_save" value="<?=_("Save")?>" onClick="kdbSaveEntry('UserUI', 'save', 'kdb_form_user'); return false;">
</p>
</div>
</form>
<?php
  }

  public function getHTMLRegistration(DbConnection $DB) {
    $userProperties = explode(",", KDbConfig::getUserProperties());
    $userPropertiesReq = explode(",", KDbConfig::getUserPropertiesReq());
    
    foreach($userProperties as $property) {
      $getHTML = "getHTML$property";
      $classes = array();
      if(in_array($property, $userPropertiesReq))
        $classes[] = "required";
      if(!$this->data->validation->isValid($property)) {
        $classes[] = "error";
        echo str_replace("</p>", 
                         "  <label for=\"{$property}\" class=\"invalid\">{$this->data->validation->getInvalidMessage($property)}</label>\n</p>", 
                         $this->$getHTML($classes));
      }
      else
        echo $this->$getHTML($classes);
      echo "\n";
    }
  }
  
  private function getClassString($classArray) {
    if(count($classArray)>0)
      return "class=\"".implode(" ", $classArray)."\" ";
    else
      return "";
  }
}

?>